/* global setTimeout, clearTimeout */
'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports['default'] = debounce;

function debounce(fn) {
  var wait = arguments.length <= 1 || arguments[1] === undefined ? 0 : arguments[1];

  var _ref = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  var _ref$leading = _ref.leading;
  var leading = _ref$leading === undefined ? false : _ref$leading;

  var nextArgs = undefined;
  var pending = undefined;
  var resolve = undefined;
  var reject = undefined;
  var timer = undefined;
  return function () {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    nextArgs = args;
    var onTimeout = run.bind(this, nextArgs, resolve, reject);
    if (!pending) {
      if (leading) {
        pending = fn.apply(this, nextArgs);
        onTimeout = clear;
      } else {
        pending = new Promise(function (_resolve, _reject) {
          resolve = _resolve;
          reject = _reject;
        });
      }
    }
    clearTimeout(timer);
    timer = setTimeout(onTimeout, getWait(wait));
    return pending;
  };

  function run(_nextArgs, _resolve, _reject) {
    fn.apply(this, _nextArgs).then(_resolve, _reject);
    clear();
  }

  function clear() {
    nextArgs = null;
    resolve = null;
    reject = null;
    pending = null;
    timer = null;
  }

  function getWait(_wait) {
    if (typeof _wait === 'function') {
      return _wait();
    }
    return _wait;
  }
}

module.exports = exports['default'];
//# sourceMappingURL=index.js.map